/* jslib */
include('chrome://jslib/content/io/io.js');
include('chrome://jslib/content/rdf/rdf.js');
include('chrome://jslib/content/rdf/rdfBase.js');
include('chrome://jslib/content/rdf/rdfContainer.js');
include('chrome://jslib/content/rdf/rdfFile.js');
include('chrome://jslib/content/rdf/rdfResource.js');

/* service contractids */
const PREF_CONTRACTID = "@mozilla.org/preferences;1";
const WSTRING_CONTRACTID = "@mozilla.org/supports-wstring;1";

const CLIENTVERSION = "Mozilla-Livelizard/1.2";

const MOOD_NAME = 0;
const MOOD_ID = 1;

/* pref strings */
const USERNAME_PREF = "livelizard.username";
const HPASSWORD_PREF = "livelizard.hpassword";
const JOURNAL_URL_PREF = "livelizard.journalUrl";
const DISPLAYMODE_PREF = "livelizard.displayMode";

var request;
var statusbarText;

/* cached data */
var moods = [];

function Init() {
  /* Need to set defaults before mucking with prefs.
   * This should be moved into a package init function,
   * does one exist? */
  getPref().SetDefaultCharPref(USERNAME_PREF, "");
  getPref().SetDefaultCharPref(HPASSWORD_PREF, "");
  getPref().SetDefaultCharPref(JOURNAL_URL_PREF,
    "http://www.livejournal.com/interface/flat");
  getPref().SetDefaultCharPref(DISPLAYMODE_PREF, "0");

  var username = getPref().CopyCharPref(USERNAME_PREF);
  var hpassword = getPref().CopyCharPref(HPASSWORD_PREF);

  statusbarText = document.getElementById("statusbar-text");
  updateDate();
  loadMoods();

  if (username.length > 0) {
    document.getElementById("username").value = username;
  }

  if (hpassword.length > 0) {
    document.getElementById("password").value = "****************";
    document.getElementById("hpassword").value = hpassword;
  }

  if (username.length > 0 && hpassword.length > 0) {
    login();
  }
}

function Destroy() {
  saveCredentials();
}

function login() {
  statusbarText.value = "Logging in...";
  var username = document.getElementById("username").value;
  var hpassword = document.getElementById("hpassword").value;

  var body = "mode=login";
  body += "&user=" + escape(username);
  body += "&hpassword=" + escape(hpassword);
  body += "&getpickws=1";
  body += "&getpickwurls=1";
  body += "&getmoods=" + escape(moods.length);
  body += "&clientversion=" + escape(CLIENTVERSION);

  sendRequest(body, loginResponseHandler);
}

function loginResponseHandler(e) {
  statusbarText.value = "Login response recieved";
  if (request.responseText) {
    var response = request.responseText.split("\n");
    var success = getResponseData("success", response);
    var message = getResponseData("message", response);

    if (success == "OK") {
      var i;

      statusbarText.value = "Loading shared journals";
      var usejournal_item;
      var usejournal_menulist = document.getElementById("usejournal"); 

      usejournal_menulist.removeChild(usejournal_menulist.firstChild);
      usejournal_menulist.appendChild(document.createElement("menupopup"));
      usejournal_menulist.appendItem(document.getElementById("username").value);
      for (i = 1; i <= getResponseData("access_count", response); i++) {
        usejournal_item = getResponseData("access_" + i, response);
        usejournal_menulist.appendItem(usejournal_item);
      }
      usejournal_menulist.selectedIndex = 0;

      statusbarText.value = "Loading pictures";
      var pickw, pickw_url;
      var pickw_menulist = document.getElementById("pickw");
      pickw_menulist.removeChild(pickw_menulist.firstChild);
      pickw_menulist.appendChild(document.createElement("menupopup"));
      pickw_menulist.appendItem("(default)");
      for (i = 1; i <= getResponseData("pickw_count", response); i++) {
        pickw = getResponseData("pickw_" + i, response);
        pickw_url = getResponseData("pickwurl_" + i, response);
        pickw_menulist.appendItem(pickw, pickw).src = pickw_url;
      }

      statusbarText.value = "Loading moods";
      var mood_id, mood_name;
      var mood_menulist = document.getElementById("mood");
      for (i = 1; i <= getResponseData("mood_count", response); i++) {
        mood_id = getResponseData("mood_" + i + "_id", response);
        mood_name = getResponseData("mood_" + i + "_name", response);

        moods[i] = [mood_name, mood_id];

        mood_menulist.appendItem(mood_name, mood_id);
      }

      statusbarText.value = "Login successful: " +
        getResponseData("name", response);

      saveCredentials();
      saveMoods();
    } else {
      statusbarText.value = "Login failed: " +
        getResponseData("errmsg", response);
    }

    if (message) {
      alert(message);
    }
  } else {
    statusbarText.value = "Login failed: Unable to connect to server";
  }
}

function postevent() {
  var username = document.getElementById("username").value;
  var hpassword = document.getElementById("hpassword").value;

  var usejournal = document.getElementById("usejournal").label; 
  var subject = document.getElementById("subject").value;
  var event = document.getElementById("event").value;

  var pickw = document.getElementById("pickw").value;
  var mood = document.getElementById("mood").value;
  var music = document.getElementById("music").value;

  var autoformat = document.getElementById("autoformat").checked ? 0 : 1;
  var allowcomments = document.getElementById("allowcomments").checked ? 0 : 1;
  var backdate = document.getElementById("backdate").checked ? 1 : 0;

  var date, time;
  if (document.getElementById("autodate").checked) {
    var now = new Date();
    date = (now.getYear() + 1900) + "/" +
      (now.getMonth() + 1) + "/" + now.getDate();
    time = now.toLocaleTimeString().substring(0, now.toLocaleTimeString().lastIndexOf(":"));
  } else {
    date = document.getElementById("date").value;
    time = document.getElementById("time").value;
  }

  var year = /^(\d+)\//.exec(date)[1];
  var month = /\/(\d+)\//.exec(date)[1];
  var day = /\/(\d+)$/.exec(date)[1];

  var hour = /^(\d+):/.exec(time)[1];
  var min = /:(\d+)$/.exec(time)[1];

  var body = "mode=postevent";
  body += "&user=" + escape(username);
  if (username != usejournal) {
    body += "&usejournal=" + escape(usejournal);
  }
  body += "&hpassword=" + escape(hpassword);

  body += "&subject=" + escape(subject);
  body += "&event=" + escape(event);

  body += "&year=" + escape(year);
  body += "&mon=" + escape(month);
  body += "&day=" + escape(day);
  body += "&hour=" + escape(hour);
  body += "&min=" + escape(min);

  body += "&prop_picture_keyword=" + escape(pickw);
  if (typeof(mood) == "number") {
    body += "&prop_current_moodid=" + escape(mood);
  } else if (mood != "(None or custom)") {
    body += "&prop_current_mood=" + escape(mood);
  }
  body += "&prop_current_music=" + escape(music);

  body += "&prop_opt_backdated=" + escape(backdate);
  body += "&prop_opt_preformatted=" + escape(autoformat);
  body += "&prop_opt_nocomments=" + escape(allowcomments);

  sendRequest(body, posteventResponseHandler);
}

function posteventResponseHandler(e) {
  if (request.responseText) {
    var response = request.responseText.split("\n");
    var success = getResponseData("success", response);
    var message = getResponseData("message", response);

    if (success == "OK") {
      statusbarText.value = "Update successful";
      updateDate();
      document.getElementById("event").value = "";
      document.getElementById("subject").value = "";

      saveCredentials();
    } else {
      statusbarText.value = "Update failed: " +
        getResponseData("errmsg", response);
    }

    if (message) {
      alert(message);
    }
  } else {
    statusbarText.value = "Update failed: Unable to connect to server";
  }
}

/* utility functions */
function sendRequest(data, fun) {
  var body = Components.classes[WSTRING_CONTRACTID].createInstance(Components.interfaces.nsISupportsWString);
  body.data = data;

  request = new XMLHttpRequest();
  request.onload = fun;
  request.open("POST", getPref().CopyCharPref(JOURNAL_URL_PREF));
  request.send(body);
}

function getResponseData(heading, response) {
  var i = 0;
  var data = null;

  while (i != response.length) {
    if (response[i] == heading) {
      data = response[i + 1];
      return data;
    }
    i++;
  }

  return null;
}

function getPref() {
  var pref = null;

  try {
    pref = Components.classes[PREF_CONTRACTID].getService();
    if (pref) pref = pref.QueryInterface(Components.interfaces.nsIPref);
  } catch (e) {
    debug("getPref(): caught exception \"" + e + "\" getting pref");
    throw(e);
  }

  return pref;
}

function updatehpassword() {
  document.getElementById("hpassword").value =
    hexMD5(document.getElementById("password").value);
}

function updateDate() {
  var now = new Date();
  document.getElementById("date").value = (now.getYear() + 1900) +
    "/" + (now.getMonth() + 1) + "/" + now.getDate();
  document.getElementById("time").value = now.toTimeString();
}

function loadMoods() {
}

function saveMoods() {
  var du = new DirUtils();
  var d = new Dir(du.getMozUserHomeDir());
  var file = d.append("livelizard.rdf");
  var livelizardRDF = new RDFFile(file);

  var seq = livelizardRDF.addRootSeq("urn:livelizard:moods");
  if (seq != null) {
    for (var i = 1; i <= moods.length; i++) {
      var mood = moods[i];
      var mood_id = mood[MOOD_ID];
      var mood_name = mood[MOOD_NAME];
      //alert(i + " id: " + mood_id + ", name: " + mood_name);
      var node = seq.addNode(mood_id);
      seq.setAttribute("urn:livelizard:moods:" + mood_id,
        "Name", mood_name);
    }
  }

  livelizardRDF.flush();
}

function saveCredentials() {
  getPref().SetCharPref(USERNAME_PREF,
    document.getElementById("username").value);
  getPref().SetCharPref(HPASSWORD_PREF,
    document.getElementById("hpassword").value);
}
